﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class EnemyFactory : MonoBehaviour {
	public PlayerController Player;
	public AlienBehaviour AlienPrefab;
	public Camera mainCam;

	private List<AlienBehaviour> _activeEnemies;
	private List<AlienBehaviour> _tempEnemies;

	private int _enemiesDestroyed = 0;
	private int _maxEnemies;

	// Starting defaults - resets to these upon new game
	public int StartingEnemyCount;
	public float MinAlienRespawn;
	public float MaxAlienRespawn;
	public float MinAlienSpeed;
	public float MaxAlienSpeed;

	// Current enemy stats
	private float _minRespawn;
	private float _maxRespawn;
	private float _alienSpeedMin;
	private float _alienSpeedMax;

	// Respawn variables
	private float _respawnTime = 0f;
	private float _timePassed = 0f;
	private float _spawnZoneWidth = 0f;
	private float _spawnZoneHeight = 0f;
	private float _spawnMinX = 0f;

	void Start() {
		_activeEnemies = new List<AlienBehaviour>();
		_tempEnemies = new List<AlienBehaviour>();
		ResetFactoryDefaults();
		SetSpawnZone();
	}

	/*
	 * Sets the default spawn zone area
	 */
	private void SetSpawnZone() {
		_spawnZoneWidth = mainCam.ScreenToWorldPoint(new Vector3(Screen.width*1.5f, 0f, 0f)).x;
		_spawnZoneHeight = mainCam.ScreenToWorldPoint(new Vector3(0f, Screen.height, 0f)).y + 0.5f;
		_spawnMinX = mainCam.ScreenToWorldPoint(new Vector3(10.0f, 0f, 0f)).x;
	}

	/*
	 * Resets default values - for use with New Game
	 */
	public void ResetFactoryDefaults() {
		_maxEnemies = StartingEnemyCount;
		_minRespawn = MinAlienRespawn;
		_maxRespawn = MaxAlienRespawn;
		_alienSpeedMin = MinAlienSpeed;
		_alienSpeedMax = MaxAlienSpeed;
		_enemiesDestroyed = 0;
	}

	// Update is called once per frame
	public void UpdateFactory (MainGameControllerBehaviour.GameState currentState) {
		// If in-game, potentially spawn more enemies
		if (currentState == MainGameControllerBehaviour.GameState.InGame) {
			if ((_activeEnemies.Count < _maxEnemies) && (_timePassed > _respawnTime)) {
				SpawnNewEnemy();
			}

			// Only consider in-game time as passed time
			_timePassed += Time.deltaTime;
		}

		// Get enemies for iteration
		_tempEnemies.Clear();
		_tempEnemies.AddRange(_activeEnemies);

		// Update each enemy in the temp list
		foreach(AlienBehaviour enemy in _tempEnemies) {
			enemy.UpdateEnemy(currentState);

			// Destroy enemies marked for destruction
			if(enemy.ShouldBeDestroyed) {
				if (enemy.ShouldDestroyPlayer && Player.gameObject.activeSelf) {
					Player.DestroyShip();
				}
				DestroyEnemy(enemy);
			}
		}

		// Increase difficulty when 5+ enemies are killed
		if(_enemiesDestroyed >= 5) {
			IncreaseDifficulty();
		}
	}

	/*
	 * Increases the difficulty of the current match
	 */
	private void IncreaseDifficulty() {
		float modifier = 0.035f;
		_minRespawn -= modifier;
		_maxRespawn -= modifier;
		_alienSpeedMin += modifier;
		_alienSpeedMax += modifier;
		_maxEnemies++;
		_enemiesDestroyed = 0;
	}

	/*
	 * Spawn's a new enemy with randomized details
	 */
	private void SpawnNewEnemy() {
		// Randomize starting position
		int randomStartX = Random.Range((int)(_spawnMinX), (int)(_spawnMinX+_spawnZoneWidth));

		AlienBehaviour alien = (AlienBehaviour)Instantiate(AlienPrefab, 
		                                             new Vector3((float)randomStartX, _spawnZoneHeight, 0.0f),
		                                             transform.rotation);
		alien.SetRandomSpeed(_alienSpeedMin, _alienSpeedMax);
		_activeEnemies.Add(alien);

		_timePassed = 0f;
		_respawnTime = Random.Range (_minRespawn, _maxRespawn);
	}

	/*
	 * Destroys the passed in enemy
	 */
	public void DestroyEnemy(AlienBehaviour enemy) {
		_enemiesDestroyed++;
		_activeEnemies.Remove(enemy);
		enemy.DestroyShip();
	}
}