﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class MainGameControllerBehaviour : MonoBehaviour {
	public Camera mainCam; 

	// Play area bounds
	public BoxCollider2D leftWall;
	public BoxCollider2D rightWall;
	public BoxCollider2D topWall;
	public BoxCollider2D bottomWall;

	// Game systems to update
	public MainMenuBehaviour mainMenu;
	public GameOverBehaviour gameOverScreen;
	public EnemyFactory enemyFactoryInstance;
	public BulletFactory bulletFactoryInstance;
	public PlanetFactory planetFactoryInstance;
	public StarScrollingController starController;
	public GameObject pauseMenu;

	public KeyCode quitGame;

	public PlayerController Player01;

	// Potential game stats
	public enum GameState {
		MainMenu,
		InGame,
		Paused,
		GameOver
	};

	private Stack<GameState> _gameStateStack;
	private bool _isStartKeyDown = false;

	// Use this for initialization
	void Start () {
		Screen.showCursor = false;

		_gameStateStack = new Stack<GameState>();

		// Move each wall to the edge
		topWall.size = new Vector2(mainCam.ScreenToWorldPoint(new Vector3(Screen.width*2f, 0f, 0f)).x, 1f);
		topWall.gameObject.transform.position = new Vector2(0f, mainCam.ScreenToWorldPoint(new Vector3(0f, Screen.height, 0f)).y + 2.5f);
		
		bottomWall.size = new Vector2(mainCam.ScreenToWorldPoint(new Vector3(Screen.width*2f, 0f, 0f)).x, 1f);
 		bottomWall.gameObject.transform.position = new Vector2(0f, mainCam.ScreenToWorldPoint(new Vector3(0f, 0f, 0f)).y - 2.5f);

		leftWall.size = new Vector2(1f, mainCam.ScreenToWorldPoint(new Vector3(0f, Screen.height*2f, 0f)).y);
		leftWall.gameObject.transform.position = new Vector2(mainCam.ScreenToWorldPoint(new Vector3(0f, 0f, 0f)).x - 0.5f, 0f);

		rightWall.size = new Vector2(1f, mainCam.ScreenToWorldPoint(new Vector3(0f, Screen.height*2f, 0f)).y);
		rightWall.gameObject.transform.position = new Vector2(mainCam.ScreenToWorldPoint(new Vector3(Screen.width, 0f, 0f)).x + 0.5f, 0f);

		PushGameState(GameState.MainMenu);
	}
	
	// Update is called once per frame
	void Update () {
		UpdateCommonControls();

		switch(_gameStateStack.Peek()) {
		case GameState.MainMenu:
			mainMenu.UpdateMenu();
			break;
		case GameState.InGame:
			Player01.UpdatePlayerControls(GetGameState());
			enemyFactoryInstance.UpdateFactory(GetGameState());
			break;
		case GameState.Paused:
			Player01.UpdatePlayerControls(GetGameState());
			break;
		case GameState.GameOver:
			gameOverScreen.UpdateGameOverScreen();
			enemyFactoryInstance.UpdateFactory(GetGameState());
			break;
		}

		bulletFactoryInstance.UpdateFactory(GetGameState());
		starController.UpdateStars(GetGameState());
		planetFactoryInstance.UpdatePlanets(GetGameState());
	}

	/*
	 * Allows the user to use common controls regardless of game state
	 */
	private void UpdateCommonControls() {
		// Allow any time exit
		if(Input.GetKey(KeyCode.Escape)) {
			Application.Quit();
		}

		// Fixes issue with New Game going immediately to paused
		if(IsStartKeyDown) {
			if(Input.GetKeyUp(mainMenu.startButton)) {
				IsStartKeyDown = false;
			}
		}
	}

	/*
	 * Pushes the provided game state to the state stack
	 */
	public void PushGameState(GameState gState) {
		switch(gState) {
		case GameState.MainMenu:
			// Reset the game from game over
			if(_gameStateStack.Count != 0 && _gameStateStack.Peek() == GameState.GameOver) {
				_gameStateStack.Clear();
			}
			mainMenu.ShowMenu(true);
			enemyFactoryInstance.ResetFactoryDefaults();
			_gameStateStack.Push(gState);
			break;
		case GameState.InGame:
			ScoreManagerBehaviour.ResetScore();
			Player01.PlayIntroCutscene();
			_gameStateStack.Push(gState);
			break;
		case GameState.Paused:
			if (_gameStateStack.Peek() == GameState.Paused) {
				// In this case, pop the pause state to resume game
				pauseMenu.SetActive(false);
				_gameStateStack.Pop();
			} else {
				pauseMenu.SetActive(true);
				_gameStateStack.Push(gState);
			}
			break;
		case GameState.GameOver:
			_gameStateStack.Push(gState);
			break;
		}
	}

	/*
	 * Gets/Sets if the start key is down from the Main Menu
	 */
	public bool IsStartKeyDown 
	{
		get { return _isStartKeyDown; }
		set { _isStartKeyDown = value; }
	}

	/*
	 * Peeks at the current state in the game state
	 */
	public GameState GetGameState() {
		return _gameStateStack.Peek();
	}
}
